/*
 * Wireless2.4GHz.hpp
 *
 *  Created on: Apr 12, 2012
 *      Author: Andygongyb
 */

#ifndef WIRELESS_HPP_
#define WIRELESS_HPP_

#include <cstdlib>

extern "C" {

#include "intc.h"
#include "gpio.h"
#include "delay.h"

}

#include "DataBlock.hpp"
#include "SPI.hpp"
#include "Utility.hpp"
#include "IODevice.hpp"
#include "EIC.hpp"

namespace wireless_names {

const uint8_t MASK_RX_DR = 6;
const uint8_t MASK_TX_DS = 5;
const uint8_t MASK_MAX_RT = 4;
const uint8_t EN_CRC = 3;
const uint8_t CRCO = 2;
const uint8_t PWR_UP = 1;
const uint8_t PRIM_RX = 0;
const uint8_t ENAA_P5 = 5;
const uint8_t ENAA_P4 = 4;
const uint8_t ENAA_P3 = 3;
const uint8_t ENAA_P2 = 2;
const uint8_t ENAA_P1 = 1;
const uint8_t ENAA_P0 = 0;
const uint8_t ERX_P5 = 5;
const uint8_t ERX_P4 = 4;
const uint8_t ERX_P3 = 3;
const uint8_t ERX_P2 = 2;
const uint8_t ERX_P1 = 1;
const uint8_t ERX_P0 = 0;
const uint8_t AW = 0;
const uint8_t ARD = 4;
const uint8_t ARC = 0;
const uint8_t CONT_WAVE = 7;
const uint8_t RF_DR_LOW = 5;
const uint8_t PLL_LOCK = 4;
const uint8_t RF_DR_HIGH = 3;
const uint8_t RF_PWR1 = 2;
const uint8_t RF_PWR0 = 1;
const uint8_t RX_DR = 6;
const uint8_t TX_DS = 5;
const uint8_t MAX_RT = 4;
const uint8_t TX_REUSE = 6;
const uint8_t TX_FULL = 5;
const uint8_t TX_EMPTY = 4;
const uint8_t RX_FULL = 1;
const uint8_t RX_EMPTY = 0;
const uint8_t EN_DPL = 2;
const uint8_t EN_ACK_PAY = 1;
const uint8_t EN_DYN_ACK = 0;

const uint8_t PLOS_CNT_MASK = 0xF0;
const uint8_t ARC_CNT_MASK = 0x0F;

const uint8_t R_REGISTER = 0x00;
const uint8_t W_REGISTER = 0x20;
const uint8_t REGISTER_MASK = 0x1F;
const uint8_t RX_P_NO = 0x0E;
const uint8_t R_RX_PAYLOAD = 0x61;
const uint8_t W_TX_PAYLOAD = 0xA0;
const uint8_t R_RX_PL_WID = 0x60;
const uint8_t W_ACK_PAYLOAD = 0xA8;
const uint8_t FLUSH_TX = 0xE1;
const uint8_t FLUSH_RX = 0xE2;
const uint8_t REUSE_TX_PL = 0xE3;
const uint8_t NOP = 0xFF;

const uint8_t ce = AVR32_PIN_PD30;

void last_transfer(uint8_t *last_data = NULL, uint16_t last_len = 0);

void ce_low(uint8_t *last_data = NULL, uint16_t last_len = 0);
void ce_high(uint8_t *last_data = NULL, uint16_t last_len = 0);
void ce_high_last(uint8_t *last_data, uint16_t last_len);

}

void wireless_handler(uint8_t int_num);

typedef class CWireless: private CSPIDevice {
public:

	typedef enum {
		CONFIG = 0x00,
		EN_AA = 0x01,
		EN_RXADDR = 0x02,
		SETUP_AW = 0x03,
		SETUP_RETR = 0x04,
		RF_CH = 0x05,
		RF_SETUP = 0x06,
		STATUS = 0x07,
		OBSERVE_TX = 0x08,
		PRD = 0x09,
		RX_ADDR_P0 = 0x0A,
		RX_ADDR_P1 = 0x0B,
		RX_ADDR_P2 = 0x0C,
		RX_ADDR_P3 = 0x0D,
		RX_ADDR_P4 = 0x0E,
		RX_ADDR_P5 = 0x0F,
		TX_ADDR = 0x10,
		RX_PW_P0 = 0x11,
		RX_PW_P1 = 0x12,
		RX_PW_P2 = 0x13,
		RX_PW_P3 = 0x14,
		RX_PW_P4 = 0x15,
		RX_PW_P5 = 0x16,
		FIFO_STATUS = 0x17,
		DYNPD = 0x1C,
		FEATURE = 0x1D,
	} reg_map_t;

	typedef struct {
		uint8_t channel;
		uint8_t address[5];
		uint8_t tx_queue_size;
		uint8_t rx_queue_size;
	} property_t;

private:
	typedef class CPackage: public CData {
	private:
		uint8_t *m_buffer;
		uint8_t m_len;
		uint8_t m_pipe_id;
		volatile flag_t *m_if_done;
	public:
		CPackage(uint8_t *buffer, uint8_t len, volatile flag_t *if_done,
				uint8_t pipe_id) {
			m_buffer = buffer;
			m_len = len;
			m_pipe_id = pipe_id;
			m_if_done = if_done;
			if (m_if_done)
				*m_if_done = PENDING;
		}
		void free_buffer(void) {
			m_sub_buffer = m_buffer;
		}
		friend class CWireless;
		friend void wireless_handler(uint8_t int_num);
	}*pCPackage;

	static CWireless *m_instance;
	static property_t m_property;
	CPackage *m_tx_head;
	CPackage *m_tx_tail;
	CPackage *m_rx_head;
	CPackage *m_rx_tail;
	volatile uint32_t m_failure_count;
	volatile uint8_t m_tx_queue_len;
	volatile uint8_t m_rx_queue_len;

	CWireless(void);

	uint8_t cmd(uint8_t cmd, uint8_t *send = NULL, uint8_t len = 0,
			uint8_t *receive = NULL, volatile flag_t *the_flag = NULL,
			call_back_t after = &wireless_names::last_transfer);

	void write_payload(uint8_t *send, uint8_t len, volatile flag_t *the_flag =
			NULL);

public:
	~CWireless(void);

	static CWireless *instance(void) {
		if (!m_instance) {
			m_instance = new CWireless();
		}
		return m_instance;
	}

	void set_property(property_t &property);
	void get_property(property_t &property) {
		memcpy((void*) &property, (const void*) &m_property, sizeof(property_t));
	}

	virtual void puts(const char *str);

	virtual void putx(uint8_t *data, uint8_t len) {
		append(data, len);
	}

	uint16_t append(uint8_t *buffer, uint16_t len, volatile flag_t *if_done =
			NULL);
	uint8_t get_from_queue(uint8_t *&buffer);

	uint8_t read_reg(uint8_t address, uint8_t *data);
	uint8_t read_reg(uint8_t address, uint8_t *buffer, uint8_t len,
			volatile flag_t *the_flag = NULL) {
		for (uint8_t i = 0; i < len; i++)
			*(buffer + i) = wireless_names::NOP; // set the data in the buffer to NOP to prevent malfunction
		return cmd(wireless_names::R_REGISTER | (wireless_names::REGISTER_MASK
				& address), buffer, len, buffer, the_flag);
	}
	uint8_t write_reg(uint8_t address, uint8_t data);
	uint8_t write_reg(uint8_t address, uint8_t *buffer, uint8_t len,
			volatile flag_t *the_flag = NULL) {
		return cmd(wireless_names::W_REGISTER | (wireless_names::REGISTER_MASK
				& address), buffer, len, NULL, the_flag);
	}

	void change_address(uint8_t *address) {
		uint8_t tmp[5];
		for (uint8_t i = 5; i; i--) {
			*(tmp + i - 1) = address[5 - i]; // reverse the pipe0 address
			write_reg(RX_ADDR_P0, tmp, 5); // set the pipe0 address
			write_reg(TX_ADDR, tmp, 5); // set the TX address the same as the pipe0 address
		}
	}

	uint8_t flush_rx_queue(void);

	uint32_t get_falure_count(void) {
		return m_failure_count;
	}

	uint32_t reset_falure_count(void) {
		uint32_t ucTmp = m_failure_count;
		m_failure_count = 0;
		return ucTmp;
	}

	uint8_t get_tx_que_len(void) {
		return m_tx_queue_len;
	}
	uint8_t get_rx_que_len(void) {
		return m_rx_queue_len;
	}

	friend void wireless_handler(uint8_t int_num);
}*pCWireless;

#endif
