/*
 * servo.c
 *
 * Created: 2012/4/17 15:34:46
 *  Author: Andygongyb
 */ 

#include "servo.h"
#include <avr/io.h>
#include <avr/interrupt.h>

static uint8_t servo_high_period = 0;
static uint16_t servo_high_addition = 0;
volatile uint16_t servo_int_count = 0;
volatile uint8_t servo_updating = 0;

// @16MHz, the period if timer 1 is 0.064ms
// as a result, 19.968ms is about 312 times the period
// default left 90 degree is about at the 8th period
// default right 90 degree is about at the 35th period
#define SERVO_PERIOD 312

servo_property_t servo_property = {{8, 54}};

void servo_init(void) {
	cli();
	PORTB &= ~_BV(PB2);
	DDRB |= _BV(PB2);
	TCCR1A &= ~_BV(COM1B0) & ~_BV(COM1B1); // make sure the OC pin is disconnected
	TIMSK = (TIMSK | _BV(TOIE1)) & ~_BV(OCIE1B);
	sei();
}

void servo_set(int16_t pos) {
	pos += 256;
	if (pos < 0) {
		pos = 0;
	} else if (pos > 512) {
		pos = 512;
	}
	pos *= servo_property.property.servo_length;
	uint16_t tmp = pos;
	do {} while(servo_updating);
	cli();
	servo_high_period = tmp / 1024 + servo_property.property.servo_left;
	servo_high_addition = tmp % 1024;
	if (servo_high_addition) {
		// leave one period for high watermark
		servo_high_period --;
	}	
	sei();
}

ISR(TIMER1_OVF_vect) {
	servo_int_count++;
	if (servo_int_count == servo_high_period) {
		if (servo_high_addition) {
			OCR1B = servo_high_addition;
			TIFR |= _BV(OCF1B);
			TIMSK |= _BV(OCIE1B);
		} else {
			PORTB &= ~_BV(PB2);
			servo_updating = 0;
		}
	} else if (servo_int_count == SERVO_PERIOD) {
		servo_int_count = 0;
		servo_updating = 1;
		PORTB |= _BV(PB2);
	}		
}

ISR(TIMER1_COMPB_vect) {
	if (servo_int_count != servo_high_period) {
		PORTB &= ~_BV(PB2);
		TIMSK &= ~_BV(OCIE1B);
		servo_updating = 0;
	}
}